@@ -5,6 +5,8 @@
 #include <glib.h>
 #include <openssl/ssl.h>
 #include <openssl/x509.h>
+#include <openssl/bn.h>
+#include <openssl/rsa.h>
 #include <sys/types.h>
 #include <sys/socket.h>
 #include <openssl/err.h>
@@ -19,7 +21,10 @@
 #include "ice.h"
 
 
-
+#if OPENSSL_VERSION_NUMBER >= 0x10002000L
+#define X509_getm_notBefore X509_get_notBefore
+#define X509_getm_notAfter X509_get_notAfter
+#endif
 
 
 #define DTLS_DEBUG 0
@@ -27,7 +32,7 @@
 #if DTLS_DEBUG
 #define __DBG(x...) ilog(LOG_DEBUG, x)
 #else
-#define __DBG(x...) ((void)0)
+#define __DBG(x...) ilogs(internals, LOG_DEBUG, x)
 #endif
 
 
@@ -62,6 +67,11 @@ static unsigned int sha_512_func(unsigned char *, X509 *);
 
 
 static const struct dtls_hash_func hash_funcs[] = {
+	{
+		.name					= "sha-256",
+		.num_bytes				= 256 / 8,
+		.__func					= sha_256_func,
+	},
 	{
 		.name					= "sha-1",
 		.num_bytes				= 160 / 8,
@@ -72,11 +82,6 @@ static const struct dtls_hash_func hash_funcs[] = {
 		.num_bytes				= 224 / 8,
 		.__func					= sha_224_func,
 	},
-	{
-		.name					= "sha-256",
-		.num_bytes				= 256 / 8,
-		.__func					= sha_256_func,
-	},
 	{
 		.name					= "sha-384",
 		.num_bytes				= 384 / 8,
@@ -120,6 +125,7 @@ static void cert_free(void *p) {
 		EVP_PKEY_free(cert->pkey);
 	if (cert->x509)
 		X509_free(cert->x509);
+	g_queue_clear_full(&cert->fingerprints, free);
 }
 
 static void buf_dump_free(char *buf, size_t len) {
@@ -134,7 +140,7 @@ static void buf_dump_free(char *buf, size_t len) {
 		else
 			llen = len;
 
-		ilog(LOG_DEBUG, "--- %.*s", llen, p);
+		ilogs(srtp, LOG_DEBUG, "--- %.*s", llen, p);
 
 		if (!f)
 			break;
@@ -150,23 +156,31 @@ static void dump_cert(struct dtls_cert *cert) {
 	char *buf;
 	size_t len;
 
-	if (get_log_level() < LOG_DEBUG)
+	if (get_log_level(core) < LOG_DEBUG)
 		return;
 
 	/* cert */
 	fp = open_memstream(&buf, &len);
+	if (!fp) {
+		ilogs(crypto, LOG_ERROR, "Failed to allocate memory to dump DTLS certificate");
+		return;
+	}
 	PEM_write_X509(fp, cert->x509);
 	fclose(fp);
 
-	ilog(LOG_DEBUG, "Dump of DTLS certificate:");
+	ilogs(srtp, LOG_DEBUG, "Dump of DTLS certificate:");
 	buf_dump_free(buf, len);
 
 	/* key */
 	fp = open_memstream(&buf, &len);
+	if (!fp) {
+		ilogs(crypto, LOG_ERROR, "Failed to allocate memory to dump DTLS private key");
+		return;
+	}
 	PEM_write_PrivateKey(fp, cert->pkey, NULL, NULL, 0, 0, NULL);
 	fclose(fp);
 
-	ilog(LOG_DEBUG, "Dump of DTLS private key:");
+	ilogs(srtp, LOG_DEBUG, "Dump of DTLS private key:");
 	buf_dump_free(buf, len);
 }
 
@@ -179,7 +193,7 @@ static int cert_init(void) {
 	X509_NAME *name;
 	struct dtls_cert *new_cert;
 
-	ilog(LOG_INFO, "Generating new DTLS certificate");
+	ilogs(crypto, LOG_INFO, "Generating new DTLS certificate");
 
 	/* objects */
 
@@ -239,10 +253,10 @@ static int cert_init(void) {
 
 	/* cert lifetime */
 
-	if (!X509_gmtime_adj(X509_get_notBefore(x509), -60*60*24))
+	if (!X509_gmtime_adj(X509_getm_notBefore(x509), -60*60*24))
 		goto err;
 
-	if (!X509_gmtime_adj(X509_get_notAfter(x509), CERT_EXPIRY_TIME))
+	if (!X509_gmtime_adj(X509_getm_notAfter(x509), CERT_EXPIRY_TIME))
 		goto err;
 
 	/* sign it */
@@ -253,8 +267,13 @@ static int cert_init(void) {
 	/* digest */
 
 	new_cert = obj_alloc0("dtls_cert", sizeof(*new_cert), cert_free);
-	new_cert->fingerprint.hash_func = &hash_funcs[0];
-	dtls_fingerprint_hash(&new_cert->fingerprint, x509);
+
+	for (int i = 0; i < num_hash_funcs; i++) {
+		struct dtls_fingerprint *fp = malloc(sizeof(*fp));
+		fp->hash_func = &hash_funcs[i];
+		dtls_fingerprint_hash(fp, x509);
+		g_queue_push_tail(&new_cert->fingerprints, fp);
+	}
 
 	new_cert->x509 = x509;
 	new_cert->pkey = pkey;
@@ -281,7 +300,7 @@ static int cert_init(void) {
 	return 0;
 
 err:
-	ilog(LOG_ERROR, "Failed to generate DTLS certificate");
+	ilogs(crypto, LOG_ERROR, "Failed to generate DTLS certificate");
 
 	if (pkey)
 		EVP_PKEY_free(pkey);
@@ -420,7 +439,7 @@ static int verify_callback(int ok, X509_STORE_CTX *store) {
 		X509_free(ps->dtls_cert);
 	ps->dtls_cert = X509_dup(X509_STORE_CTX_get_current_cert(store));
 
-	if (!media->fingerprint.hash_func)
+	if (!media->fingerprint.hash_func || !media->fingerprint.digest_len)
 		return 1; /* delay verification */
 
 	if (dtls_verify_cert(ps))
@@ -441,7 +460,7 @@ int dtls_verify_cert(struct packet_stream *ps) {
 	dtls_hash(media->fingerprint.hash_func, ps->dtls_cert, fp);
 
 	if (memcmp(media->fingerprint.digest, fp, media->fingerprint.hash_func->num_bytes)) {
-		ilog(LOG_WARNING, "DTLS: Peer certificate rejected - fingerprint mismatch");
+		ilogs(crypto, LOG_WARNING, "DTLS: Peer certificate rejected - fingerprint mismatch");
 		__DBG("fingerprint expected: %02x%02x%02x%02x%02x%02x%02x%02x received: %02x%02x%02x%02x%02x%02x%02x%02x",
 			media->fingerprint.digest[0], media->fingerprint.digest[1],
 			media->fingerprint.digest[2], media->fingerprint.digest[3],
@@ -453,7 +472,7 @@ int dtls_verify_cert(struct packet_stream *ps) {
 	}
 
 	PS_SET(ps, FINGERPRINT_VERIFIED);
-	ilog(LOG_INFO, "DTLS: Peer certificate accepted");
+	ilogs(crypto, LOG_INFO, "DTLS: Peer certificate accepted");
 
 	return 0;
 }
@@ -476,7 +495,7 @@ static int try_connect(struct dtls_connection *d) {
 	ret = 0;
 	switch (code) {
 		case SSL_ERROR_NONE:
-			ilog(LOG_DEBUG, "DTLS handshake successful");
+			ilogs(crypto, LOG_DEBUG, "DTLS handshake successful");
 			d->connected = 1;
 			ret = 1;
 			break;
@@ -487,7 +506,7 @@ static int try_connect(struct dtls_connection *d) {
 
 		default:
 			ret = ERR_peek_last_error();
-			ilog(LOG_ERROR, "DTLS error: %i (%s)", code, ERR_reason_error_string(ret));
+			ilogs(crypto, LOG_ERROR, "DTLS error: %i (%s)", code, ERR_reason_error_string(ret));
 			ret = -1;
 			break;
 	}
@@ -500,8 +519,6 @@ int dtls_connection_init(struct dtls_connection *d, struct packet_stream *ps, in
 {
 	unsigned long err;
 
-	__DBG("dtls_connection_init(%i)", active);
-
 	if (d->init) {
 		if ((d->active && active) || (!d->active && !active))
 			goto done;
@@ -510,6 +527,8 @@ int dtls_connection_init(struct dtls_connection *d, struct packet_stream *ps, in
 
 	d->ptr = ps;
 
+	ilogs(crypto, LOG_DEBUG, "Creating %s DTLS connection context", active ? "active" : "passive");
+
 #if OPENSSL_VERSION_NUMBER >= 0x10002000L
 	d->ssl_ctx = SSL_CTX_new(active ? DTLS_client_method() : DTLS_server_method());
 #else
@@ -556,9 +575,9 @@ int dtls_connection_init(struct dtls_connection *d, struct packet_stream *ps, in
 
 #if defined(SSL_OP_NO_QUERY_MTU)
 	SSL_CTX_set_options(d->ssl_ctx, SSL_OP_NO_QUERY_MTU);
-	SSL_set_mtu(d->ssl, 1500);
+	SSL_set_mtu(d->ssl, rtpe_config.dtls_mtu);
 #if defined(DTLS_set_link_mtu) || defined(DTLS_CTRL_SET_LINK_MTU) || OPENSSL_VERSION_NUMBER >= 0x10100000L
-	DTLS_set_link_mtu(d->ssl, 1500);
+	DTLS_set_link_mtu(d->ssl, rtpe_config.dtls_mtu);
 #endif
 #endif
 
@@ -580,7 +599,7 @@ error:
 	if (d->ssl_ctx)
 		SSL_CTX_free(d->ssl_ctx);
 	ZERO(*d);
-	ilog(LOG_ERROR, "Failed to init DTLS connection: %s", ERR_reason_error_string(err));
+	ilogs(crypto, LOG_ERROR, "Failed to init DTLS connection: %s", ERR_reason_error_string(err));
 	return -1;
 }
 
@@ -630,7 +649,7 @@ found:
 	i += cs->master_salt_len;
 	memcpy(server.master_salt, &keys[i], cs->master_salt_len);
 
-	ilog(LOG_INFO, "DTLS-SRTP successfully negotiated");
+	ilogs(crypto, LOG_INFO, "DTLS-SRTP successfully negotiated using %s", cs->name);
 
 	if (d->active) {
 		/* we're the client */
@@ -660,9 +679,9 @@ found:
 
 error:
 	if (!spp)
-		ilog(LOG_ERROR, "Failed to set up SRTP after DTLS negotiation: %s", err);
+		ilogs(crypto, LOG_ERROR, "Failed to set up SRTP after DTLS negotiation: %s", err);
 	else
-		ilog(LOG_ERROR, "Failed to set up SRTP after DTLS negotiation: %s (profile \"%s\")",
+		ilogs(crypto, LOG_ERROR, "Failed to set up SRTP after DTLS negotiation: %s (profile \"%s\")",
 				err, spp->name);
 	return -1;
 }
@@ -682,7 +701,7 @@ int dtls(struct stream_fd *sfd, const str *s, const endpoint_t *fsin) {
 		return 0;
 
 	if (s)
-		__DBG("dtls packet input: len %u %02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x",
+		__DBG("dtls packet input: len %zu %02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x",
 			s->len,
 			(unsigned char) s->s[0], (unsigned char) s->s[1], (unsigned char) s->s[2], (unsigned char) s->s[3],
 			(unsigned char) s->s[4], (unsigned char) s->s[5], (unsigned char) s->s[6], (unsigned char) s->s[7],
@@ -693,7 +712,7 @@ int dtls(struct stream_fd *sfd, const str *s, const endpoint_t *fsin) {
 		return -1;
 
 	if (s) {
-		ilog(LOG_DEBUG, "Processing incoming DTLS packet");
+		ilogs(srtp, LOG_DEBUG, "Processing incoming DTLS packet");
 		BIO_write(d->r_bio, s->s, s->len);
 		/* we understand this as preference of DTLS over SDES */
 		MEDIA_CLEAR(ps->media, SDES);
@@ -701,7 +720,7 @@ int dtls(struct stream_fd *sfd, const str *s, const endpoint_t *fsin) {
 
 	ret = try_connect(d);
 	if (ret == -1) {
-		ilog(LOG_ERROR, "DTLS error on local port %u", sfd->socket.local.port);
+		ilogs(srtp, LOG_ERROR, "DTLS error on local port %u", sfd->socket.local.port);
 		/* fatal error */
 		dtls_connection_cleanup(d);
 		return 0;
@@ -710,7 +729,7 @@ int dtls(struct stream_fd *sfd, const str *s, const endpoint_t *fsin) {
 		/* connected! */
 		mutex_lock(&ps->out_lock); // nested lock!
 		if (dtls_setup_crypto(ps, d))
-			/* XXX ?? */ ;
+			{} /* XXX ?? */
 		mutex_unlock(&ps->out_lock);
 
 		if (PS_ISSET(ps, RTP) && PS_ISSET(ps, RTCP) && ps->rtcp_sibling
@@ -721,7 +740,7 @@ int dtls(struct stream_fd *sfd, const str *s, const endpoint_t *fsin) {
 			mutex_lock(&ps->rtcp_sibling->in_lock);
 			mutex_lock(&ps->rtcp_sibling->out_lock);
 			if (dtls_setup_crypto(ps->rtcp_sibling, d))
-				/* XXX ?? */ ;
+				{} /* XXX ?? */
 			mutex_unlock(&ps->rtcp_sibling->out_lock);
 			mutex_unlock(&ps->rtcp_sibling->in_lock);
 		}
@@ -733,7 +752,7 @@ int dtls(struct stream_fd *sfd, const str *s, const endpoint_t *fsin) {
 			break;
 
 		if (ret > sizeof(buf)) {
-			ilog(LOG_ERROR, "BIO buffer overflow");
+			ilogs(srtp, LOG_ERROR, "BIO buffer overflow");
 			(void) BIO_reset(d->w_bio);
 			break;
 		}
@@ -752,7 +771,7 @@ int dtls(struct stream_fd *sfd, const str *s, const endpoint_t *fsin) {
 		if (!fsin)
 			fsin = &ps->endpoint;
 
-		ilog(LOG_DEBUG, "Sending DTLS packet");
+		ilogs(srtp, LOG_DEBUG, "Sending DTLS packet");
 		socket_sendto(&sfd->socket, buf, ret, fsin);
 	}
 
@@ -797,11 +816,12 @@ void dtls_shutdown(struct packet_stream *ps) {
 		ps->dtls_cert = NULL;
 	}
 
-	crypto_reset(&ps->crypto);
+	call_stream_crypto_reset(ps);
 }
 
 void dtls_connection_cleanup(struct dtls_connection *c) {
-	__DBG("dtls_connection_cleanup");
+	if (c->ssl_ctx || c->ssl)
+		ilogs(crypto, LOG_DEBUG, "Resetting DTLS connection context");
 
 	if (c->ssl_ctx)
 		SSL_CTX_free(c->ssl_ctx);
