@@ -1,5 +1,7 @@
 #include <errno.h>
 #include "dtmf.h"
+#include "bencode.h"
+#include "control_ng.h"
 #include "media_socket.h"
 #include "log.h"
 #include "call.h"
@@ -9,17 +11,57 @@
 #include "codec.h"
 #include "ssrc.h"
 
-
-
 static socket_t dtmf_log_sock;
 
 void dtmf_init(void) {
+	ilog(LOG_DEBUG, "log dtmf over ng %d", rtpe_config.dtmf_via_ng);
 	if (rtpe_config.dtmf_udp_ep.port) {
 		if (connect_socket(&dtmf_log_sock, SOCK_DGRAM, &rtpe_config.dtmf_udp_ep))
 			ilog(LOG_ERR, "Failed to open/connect DTMF logging socket: %s", strerror(errno));
 	}
 }
 
+static void dtmf_bencode_and_notify(struct media_packet *mp,
+		struct telephone_event_payload *dtmf, int clockrate)
+{
+	bencode_buffer_t bencbuf;
+	bencode_item_t *notify, *data, *tags;
+	str encoded_data;
+	int ret = bencode_buffer_init(&bencbuf);
+	assert(ret == 0);
+
+	if (!dtmf->end)
+		return;
+
+	if (!clockrate)
+		clockrate = 8000;
+
+	notify = bencode_dictionary(&bencbuf);
+	bencode_dictionary_add_string(notify, "notify", "onDTMF");
+	data = bencode_dictionary_add_dictionary(notify, "data");
+	tags = bencode_dictionary_add_list(data, "tags");
+
+	bencode_dictionary_add_string_len(data, "callid", mp->call->callid.s, mp->call->callid.len);
+	bencode_dictionary_add_string_len(data, "source_tag", mp->media->monologue->tag.s, mp->media->monologue->tag.len);
+
+	GList *tag_values = g_hash_table_get_values(mp->call->tags);
+	for (GList *tag_it = tag_values; tag_it; tag_it = tag_it->next) {
+		struct call_monologue *ml = tag_it->data;
+		bencode_list_add_str(tags, &ml->tag);
+	}
+	g_list_free(tag_values);
+
+	bencode_dictionary_add_string(data, "type", "DTMF");
+	bencode_dictionary_add_string(data, "source_ip", sockaddr_print_buf(&mp->fsin.address));
+	bencode_dictionary_add_integer(data, "timestamp", rtpe_now.tv_sec);
+	bencode_dictionary_add_integer(data, "event", dtmf->event);
+	bencode_dictionary_add_integer(data, "duration", ((long long) ntohs(dtmf->duration) * (1000000LL / clockrate)) / 1000LL);
+	bencode_dictionary_add_integer(data, "volume", dtmf->volume);
+
+	bencode_collapse_str(notify, &encoded_data);
+	notify_ng_tcp_clients(&encoded_data);
+	bencode_buffer_free(&bencbuf);
+}
 
 static GString *dtmf_json_print(struct media_packet *mp,
 		struct telephone_event_payload *dtmf, int clockrate)
@@ -64,7 +106,7 @@ static GString *dtmf_json_print(struct media_packet *mp,
 }
 
 int dtmf_do_logging(void) {
-	if (_log_facility_dtmf || dtmf_log_sock.family)
+	if (_log_facility_dtmf || dtmf_log_sock.family || rtpe_config.dtmf_via_ng)
 		return 1;
 	return 0;
 }
@@ -72,7 +114,7 @@ int dtmf_do_logging(void) {
 int dtmf_event(struct media_packet *mp, str *payload, int clockrate) {
 	struct telephone_event_payload *dtmf;
 	if (payload->len < sizeof(*dtmf)) {
-		ilog(LOG_WARN | LOG_FLAG_LIMIT, "Short DTMF event packet (len %u)", payload->len);
+		ilog(LOG_WARN | LOG_FLAG_LIMIT, "Short DTMF event packet (len %zu)", payload->len);
 		return -1;
 	}
 	dtmf = (void *) payload->s;
@@ -80,7 +122,7 @@ int dtmf_event(struct media_packet *mp, str *payload, int clockrate) {
 	ilog(LOG_DEBUG, "DTMF event: event %u, volume %u, end %u, duration %u",
 			dtmf->event, dtmf->volume, dtmf->end, dtmf->duration);
 
-	int ret = 0;
+	int ret = dtmf->end ? 1 : 0;
 
 	GString *buf = NULL;
 
@@ -91,10 +133,13 @@ int dtmf_event(struct media_packet *mp, str *payload, int clockrate) {
 		if (_log_facility_dtmf)
 			dtmflog(buf);
 		if (dtmf_log_sock.family)
-			send(dtmf_log_sock.fd, buf->str, buf->len, 0);
-		g_string_free(buf, TRUE);
+			if (send(dtmf_log_sock.fd, buf->str, buf->len, 0) < 0)
+				ilog(LOG_ERR, "Error sending DTMF event info to UDP socket: %s",
+						strerror(errno));
 
-		ret = 1; // END event
+		if (rtpe_config.dtmf_via_ng)
+			dtmf_bencode_and_notify(mp, dtmf, clockrate);
+		g_string_free(buf, TRUE);
 	}
 
 	return ret;
@@ -190,7 +235,11 @@ static const char *dtmf_inject_pcm(struct call_media *media, struct call_monolog
 {
 	struct call *call = monologue->call;
 
-	struct ssrc_ctx *ssrc_out = get_ssrc_ctx(ssrc_in->ssrc_map_out, call->ssrc_hash, SSRC_DIR_OUTPUT,
+	if (!media->monologue || !media->monologue->active_dialogue)
+		return "No dialogue association";
+
+	struct ssrc_ctx *ssrc_out = get_ssrc_ctx(ssrc_in->ssrc_map_out,
+			media->monologue->active_dialogue->ssrc_hash, SSRC_DIR_OUTPUT,
 			monologue);
 	if (!ssrc_out)
 		return "No output SSRC context present"; // XXX generate stream
@@ -216,6 +265,7 @@ static const char *dtmf_inject_pcm(struct call_media *media, struct call_monolog
 		.tv = rtpe_now,
 		.call = call,
 		.media = media,
+		.media_out = media,
 		.rtp = &rtp,
 		.ssrc_in = ssrc_in,
 		.ssrc_out = ssrc_out,
@@ -270,8 +320,6 @@ const char *dtmf_inject(struct call_media *media, int code, int volume, int dura
 		pt = ssrc_in->tracker.most[i];
 		if (pt == 255)
 			continue;
-		if (pt < 0)
-			break;
 
 		ch = codec_handler_get(media, pt);
 		if (!ch)
@@ -313,7 +361,7 @@ const char *dtmf_inject(struct call_media *media, int code, int volume, int dura
 			ssrc_in->parent->h.ssrc);
 
 	// synthesise start and stop events
-	uint64_t num_samples = duration * ch->dest_pt.clock_rate / 1000;
+	uint64_t num_samples = (uint64_t) duration * ch->dest_pt.clock_rate / 1000;
 	uint64_t start_pts = codec_encoder_pts(csh);
 	uint64_t last_end_pts = codec_last_dtmf_event(csh);
 	if (last_end_pts) {
