@@ -25,7 +25,7 @@
 
 #ifdef WITH_TRANSCODING
 static struct timerthread media_player_thread;
-static MYSQL __thread *mysql_conn;
+static __thread MYSQL *mysql_conn;
 
 static void media_player_read_packet(struct media_player *mp);
 #endif
@@ -43,6 +43,8 @@ static void send_timer_send_lock(struct send_timer *st, struct codec_packet *cp)
 #ifdef WITH_TRANSCODING
 // called with call->master lock in W
 static unsigned int send_timer_flush(struct send_timer *st, void *ptr) {
+	if (!st)
+		return 0;
 	return timerthread_queue_flush(&st->ttq, ptr);
 }
 
@@ -52,7 +54,7 @@ static void media_player_shutdown(struct media_player *mp) {
 	if (!mp)
 		return;
 
-	ilog(LOG_DEBUG, "shutting down media_player");
+	//ilog(LOG_DEBUG, "shutting down media_player");
 	timerthread_obj_deschedule(&mp->tt_obj);
 	mp->next_run.tv_sec = 0;
 	avformat_close_input(&mp->fmtctx);
@@ -90,12 +92,13 @@ void media_player_stop(struct media_player *mp) {
 static void __media_player_free(void *p) {
 	struct media_player *mp = p;
 
-	ilog(LOG_DEBUG, "freeing media_player");
+	//ilog(LOG_DEBUG, "freeing media_player");
 
 	media_player_shutdown(mp);
 	ssrc_ctx_put(&mp->ssrc_out);
 	mutex_destroy(&mp->lock);
 	obj_put(mp->call);
+	av_packet_free(&mp->pkt);
 }
 #endif
 
@@ -103,27 +106,28 @@ static void __media_player_free(void *p) {
 // call->master_lock held in W
 struct media_player *media_player_new(struct call_monologue *ml) {
 #ifdef WITH_TRANSCODING
-	ilog(LOG_DEBUG, "creating media_player");
+	//ilog(LOG_DEBUG, "creating media_player");
 
 	uint32_t ssrc = 0;
 	while (ssrc == 0)
-		ssrc = random();
-	struct ssrc_ctx *ssrc_ctx = get_ssrc_ctx(ssrc, ml->call->ssrc_hash, SSRC_DIR_OUTPUT, ml);
+		ssrc = ssl_random();
+	struct ssrc_ctx *ssrc_ctx = get_ssrc_ctx(ssrc, ml->ssrc_hash, SSRC_DIR_OUTPUT, ml);
 	ssrc_ctx->next_rtcp = rtpe_now;
 
 	struct media_player *mp = obj_alloc0("media_player", sizeof(*mp), __media_player_free);
 
 	mp->tt_obj.tt = &media_player_thread;
 	mutex_init(&mp->lock);
+
 	mp->run_func = media_player_read_packet; // default
 	mp->call = obj_get(ml->call);
 	mp->ml = ml;
-	mp->seq = random();
+	mp->seq = ssl_random();
 	mp->ssrc_out = ssrc_ctx;
 
-	av_init_packet(&mp->pkt);
-	mp->pkt.data = NULL;
-	mp->pkt.size = 0;
+	mp->pkt = av_packet_alloc();
+	mp->pkt->data = NULL;
+	mp->pkt->size = 0;
 
 	return mp;
 #else
@@ -135,7 +139,7 @@ struct media_player *media_player_new(struct call_monologue *ml) {
 static void __send_timer_free(void *p) {
 	struct send_timer *st = p;
 
-	ilog(LOG_DEBUG, "freeing send_timer");
+	//ilog(LOG_DEBUG, "freeing send_timer");
 
 	obj_put(st->call);
 }
@@ -150,7 +154,7 @@ static void __send_timer_send_later(struct timerthread_queue *ttq, void *p) {
 
 // call->master_lock held in W
 struct send_timer *send_timer_new(struct packet_stream *ps) {
-	ilog(LOG_DEBUG, "creating send_timer");
+	//ilog(LOG_DEBUG, "creating send_timer");
 
 	struct send_timer *st = timerthread_queue_new("send_timer", sizeof(*st),
 			&send_timer_thread,
@@ -163,51 +167,17 @@ struct send_timer *send_timer_new(struct packet_stream *ps) {
 	return st;
 }
 
-
-// call is locked in R
-static void send_timer_send_rtcp(struct ssrc_ctx *ssrc_out, struct call *call, struct packet_stream *ps) {
-	GQueue rrs = G_QUEUE_INIT;
-	rtcp_receiver_reports(&rrs, call->ssrc_hash, ps->media->monologue);
-
-	ilog(LOG_DEBUG, "Generating and sending RTCP SR for %x and up to %i source(s)",
-			ssrc_out->parent->h.ssrc, rrs.length);
-
-	GString *sr = rtcp_sender_report(ssrc_out->parent->h.ssrc,
-			atomic64_get(&ssrc_out->last_ts),
-			atomic64_get(&ssrc_out->packets),
-			atomic64_get(&ssrc_out->octets),
-			&rrs);
-
-	socket_sendto(&ps->selected_sfd->socket, sr->str, sr->len, &ps->endpoint);
-	g_string_free(sr, TRUE);
-}
-
 // call is locked in R
 static void send_timer_rtcp(struct send_timer *st, struct ssrc_ctx *ssrc_out) {
 	struct call_media *media = st->sink ? st->sink->media : NULL;
 	if (!media)
 		return;
-	struct call *call = media->call;
 
-	// figure out where to send it
-	struct packet_stream *ps = media->streams.head->data;
-	if (MEDIA_ISSET(media, RTCP_MUX))
-		;
-	else if (!media->streams.head->next)
-		;
-	else {
-		struct packet_stream *next_ps = media->streams.head->next->data;
-		if (PS_ISSET(next_ps, RTCP))
-			ps = next_ps;
-	}
-
-	log_info_stream_fd(ps->selected_sfd);
-
-	send_timer_send_rtcp(ssrc_out, call, ps);
+	rtcp_send_report(media, ssrc_out);
 
 	// XXX missing locking?
 	ssrc_out->next_rtcp = rtpe_now;
-	timeval_add_usec(&ssrc_out->next_rtcp, 5000000);
+	timeval_add_usec(&ssrc_out->next_rtcp, 5000000 + (ssl_random() % 2000000));
 }
 
 
@@ -215,6 +185,8 @@ static void __send_timer_send_common(struct send_timer *st, struct codec_packet
 	if (!st->sink->selected_sfd)
 		goto out;
 
+	log_info_stream_fd(st->sink->selected_sfd);
+
 	struct rtp_header *rh = cp->rtp;
 	if (rh)
 		ilog(LOG_DEBUG, "Forward to sink endpoint: %s%s:%d%s (RTP seq %u TS %u)",
@@ -324,7 +296,7 @@ found:
 		}
 	}
 	if (!mp->handler)
-		mp->handler = codec_handler_make_playback(src_pt, dst_pt, mp->sync_ts);
+		mp->handler = codec_handler_make_playback(src_pt, dst_pt, mp->sync_ts, mp->media);
 	if (!mp->handler)
 		return -1;
 
@@ -377,6 +349,7 @@ void media_player_add_packet(struct media_player *mp, char *buf, size_t len,
 		.tv = rtpe_now,
 		.call = mp->call,
 		.media = mp->media,
+		.media_out = mp->media,
 		.rtp = &rtp,
 		.ssrc_out = mp->ssrc_out,
 	};
@@ -416,14 +389,31 @@ static void media_player_read_packet(struct media_player *mp) {
 	if (!mp->fmtctx)
 		return;
 
-	int ret = av_read_frame(mp->fmtctx, &mp->pkt);
+	int ret = av_read_frame(mp->fmtctx, mp->pkt);
 	if (ret < 0) {
 		if (ret == AVERROR_EOF) {
-			ilog(LOG_DEBUG, "EOF reading from media stream");
+			if (mp->repeat > 1){
+				ilog(LOG_DEBUG, "EOF reading from media stream but will repeat %li time",mp->repeat);
+				mp->repeat = mp->repeat - 1;
+				int64_t ret64 = avio_seek(mp->fmtctx->pb, 0, SEEK_SET);
+				if (ret64 != 0)
+					ilog(LOG_ERR, "Failed to seek to beginning of media file");
+				ret = av_seek_frame(mp->fmtctx, -1, 0, 0);
+				if (ret < 0)
+					ilog(LOG_ERR, "Failed to seek to beginning of media file");
+				ret = av_read_frame(mp->fmtctx, mp->pkt);
+			} else {
+				ilog(LOG_DEBUG, "EOF reading from media stream");
+				return;
+
+			}
+
+		}
+		if (ret < 0 && ret != AVERROR_EOF) { 
+			ilog(LOG_ERR, "Error while reading from media stream");
 			return;
 		}
-		ilog(LOG_ERR, "Error while reading from media stream");
-		return;
+
 	}
 
 	if (!mp->fmtctx->streams) {
@@ -442,26 +432,26 @@ static void media_player_read_packet(struct media_player *mp) {
 
 	// scale pts and duration according to sample rate
 
-	long long duration_scaled = mp->pkt.duration * avs->CODECPAR->sample_rate
+	long long duration_scaled = mp->pkt->duration * avs->CODECPAR->sample_rate
 		* avs->time_base.num / avs->time_base.den;
-	unsigned long long pts_scaled = mp->pkt.pts * avs->CODECPAR->sample_rate
+	unsigned long long pts_scaled = mp->pkt->pts * avs->CODECPAR->sample_rate
 		* avs->time_base.num / avs->time_base.den;
 
-	long long us_dur = mp->pkt.duration * 1000000LL * avs->time_base.num / avs->time_base.den;
+	long long us_dur = mp->pkt->duration * 1000000LL * avs->time_base.num / avs->time_base.den;
 	ilog(LOG_DEBUG, "read media packet: pts %llu duration %lli (scaled %llu/%lli, %lli us), "
 			"sample rate %i, time_base %i/%i",
-			(unsigned long long) mp->pkt.pts,
-			(long long) mp->pkt.duration,
+			(unsigned long long) mp->pkt->pts,
+			(long long) mp->pkt->duration,
 			pts_scaled,
 			duration_scaled,
 			us_dur,
 			avs->CODECPAR->sample_rate,
 			avs->time_base.num, avs->time_base.den);
 
-	media_player_add_packet(mp, (char *) mp->pkt.data, mp->pkt.size, us_dur, pts_scaled);
+	media_player_add_packet(mp, (char *) mp->pkt->data, mp->pkt->size, us_dur, pts_scaled);
 
 out:
-	av_packet_unref(&mp->pkt);
+	av_packet_unref(mp->pkt);
 }
 
 
@@ -504,20 +494,22 @@ found:
 
 
 // call->master_lock held in W
-static void media_player_play_start(struct media_player *mp) {
+static void media_player_play_start(struct media_player *mp, long long repeat) {
 	// needed to have usable duration for some formats. ignore errors.
 	avformat_find_stream_info(mp->fmtctx, NULL);
 
 	mp->next_run = rtpe_now;
 	// give ourselves a bit of a head start with decoding
 	timeval_add_usec(&mp->next_run, -50000);
+
 	media_player_read_packet(mp);
+	mp->repeat = repeat;
 }
 #endif
 
 
 // call->master_lock held in W
-int media_player_play_file(struct media_player *mp, const str *file) {
+int media_player_play_file(struct media_player *mp, const str *file, long long repeat) {
 #ifdef WITH_TRANSCODING
 	if (media_player_play_init(mp))
 		return -1;
@@ -531,7 +523,8 @@ int media_player_play_file(struct media_player *mp, const str *file) {
 		return -1;
 	}
 
-	media_player_play_start(mp);
+	media_player_play_start(mp,repeat);
+
 
 	return 0;
 #else
@@ -588,7 +581,7 @@ static int64_t __mp_avio_seek(void *opaque, int64_t offset, int whence) {
 
 
 // call->master_lock held in W
-int media_player_play_blob(struct media_player *mp, const str *blob) {
+int media_player_play_blob(struct media_player *mp, const str *blob, long long repeat) {
 #ifdef WITH_TRANSCODING
 	const char *err;
 	int av_ret = 0;
@@ -626,7 +619,7 @@ int media_player_play_blob(struct media_player *mp, const str *blob) {
 	if (av_ret < 0)
 		goto err;
 
-	media_player_play_start(mp);
+	media_player_play_start(mp,repeat);
 
 	return 0;
 
@@ -663,7 +656,7 @@ err:
 
 
 // call->master_lock held in W
-int media_player_play_db(struct media_player *mp, long long id) {
+int media_player_play_db(struct media_player *mp, long long id, long long repeat) {
 	const char *err;
 	AUTO_CLEANUP_GBUF(query);
 
@@ -710,7 +703,7 @@ success:;
 
 	str blob;
 	str_init_len(&blob, row[0], lengths[0]);
-	int ret = media_player_play_blob(mp, &blob);
+	int ret = media_player_play_blob(mp, &blob, repeat);
 
 	mysql_free_result(res);
 
@@ -731,7 +724,7 @@ static void media_player_run(void *ptr) {
 
 	log_info_call(call);
 
-	ilog(LOG_DEBUG, "running scheduled media_player");
+	//ilog(LOG_DEBUG, "running scheduled media_player");
 
 	rwlock_lock_r(&call->master_lock);
 	mutex_lock(&mp->lock);
@@ -763,11 +756,10 @@ void media_player_free(void) {
 
 #ifdef WITH_TRANSCODING
 void media_player_loop(void *p) {
-	ilog(LOG_DEBUG, "media_player_loop");
 	timerthread_run(&media_player_thread);
 }
 #endif
 void send_timer_loop(void *p) {
-	ilog(LOG_DEBUG, "send_timer_loop");
+	//ilog(LOG_DEBUG, "send_timer_loop");
 	timerthread_run(&send_timer_thread);
 }
