@@ -40,6 +40,10 @@
 #define MAX_RECV_ITERS 50
 #endif
 
+#ifndef MAX_RECV_LOOP_STRIKES
+#define MAX_RECV_LOOP_STRIKES 5
+#endif
+
 
 
 struct intf_rr {
@@ -532,7 +536,7 @@ got_some:
 
 	struct intf_rr *rr = g_hash_table_lookup(__logical_intf_name_family_rr_hash, &key);
 	if (!rr)
-		return __get_logical_interface(name, fam);
+		return name ? __get_logical_interface(name, fam) : log;
 	if (rr->singular) {
 		__C_DBG("Returning non-RR logical interface '" STR_FORMAT "' based on direction '" \
 					STR_FORMAT "'",
@@ -547,6 +551,8 @@ got_some:
 	log = run_round_robin_calls(rr, num_ports);
 	if (log)
 		return log;
+	if (!name)
+		return NULL;
 	return __get_logical_interface(name, fam);
 }
 static struct logical_intf *__get_logical_interface(const str *name, sockfamily_t *fam) {
@@ -1287,7 +1293,7 @@ static void __stream_update_stats(struct packet_stream *ps, int have_in_lock) {
 
 	atomic64_add(&ssrc_ctx->packets, stats.basic_stats.packets);
 	atomic64_add(&ssrc_ctx->octets, stats.basic_stats.bytes);
-	atomic64_add(&ssrc_ctx->packets_lost, stats.total_lost);
+	parent->packets_lost += stats.total_lost; // XXX should be atomic?
 	atomic64_set(&ssrc_ctx->last_seq, stats.ext_seq);
 	atomic64_set(&ssrc_ctx->last_ts, stats.timestamp);
 	parent->jitter = stats.jitter;
@@ -1987,6 +1993,7 @@ void media_packet_release(struct media_packet *mp) {
 	g_queue_clear_full(&mp->packets_out, codec_packet_free);
 	g_free(mp->rtp);
 	g_free(mp->rtcp);
+	ZERO(*mp);
 }
 
 
@@ -2189,16 +2196,26 @@ static void stream_fd_readable(int fd, void *p, uintptr_t u) {
 	struct call *ca;
 
 	if (sfd->socket.fd != fd)
-		goto out;
+		return;
 
 	log_info_stream_fd(sfd);
+	int strikes = g_atomic_int_get(&sfd->error_strikes);
+
+	if (strikes >= MAX_RECV_LOOP_STRIKES) {
+		ilog(LOG_ERROR | LOG_FLAG_LIMIT, "UDP receive queue exceeded %i times: "
+				"discarding packet", strikes);
+		// Polling is edge-triggered so we won't immediately get here again.
+		// We could remove ourselves from the poller though. Maybe call stream_fd_closed?
+		return;
+	}
 
 	for (iters = 0; ; iters++) {
 #if MAX_RECV_ITERS
 		if (iters >= MAX_RECV_ITERS) {
-			ilog(LOG_ERROR, "Too many packets in UDP receive queue (more than %d), "
+			ilog(LOG_ERROR | LOG_FLAG_LIMIT, "Too many packets in UDP receive queue (more than %d), "
 					"aborting loop. Dropped packets possible", iters);
-			break;
+			g_atomic_int_inc(&sfd->error_strikes);
+			goto strike;
 		}
 #endif
 
@@ -2218,7 +2235,7 @@ static void stream_fd_readable(int fd, void *p, uintptr_t u) {
 			goto done;
 		}
 		if (ret >= MAX_RTP_PACKET_SIZE)
-			ilog(LOG_WARNING, "UDP packet possibly truncated");
+			ilog(LOG_WARNING | LOG_FLAG_LIMIT, "UDP packet possibly truncated");
 
 		str_init_len(&phc.s, buf + RTP_BUFFER_HEAD_ROOM, ret);
 
@@ -2231,12 +2248,16 @@ static void stream_fd_readable(int fd, void *p, uintptr_t u) {
 			ret = stream_packet(&phc);
 
 		if (G_UNLIKELY(ret < 0))
-			ilog(LOG_WARNING, "Write error on media socket: %s", strerror(-ret));
+			ilog(LOG_WARNING | LOG_FLAG_LIMIT, "Write error on media socket: %s", strerror(-ret));
 		else if (phc.update)
 			update = 1;
 	}
 
-out:
+	// no strike
+	if (strikes > 0)
+		g_atomic_int_compare_and_exchange(&sfd->error_strikes, strikes, strikes - 1);
+
+strike:
 	ca = sfd->call ? : NULL;
 
 	if (ca && update) {
