@@ -52,6 +52,7 @@ static void proc_init(struct call *);
 static void sdp_before_proc(struct recording *, const str *, struct call_monologue *, enum call_opmode);
 static void sdp_after_proc(struct recording *, GString *str, struct call_monologue *, enum call_opmode opmode);
 static void meta_chunk_proc(struct recording *, const char *, const str *);
+static void update_flags_proc(struct call *call);
 static void finish_proc(struct call *);
 static void dump_packet_proc(struct media_packet *mp, const str *s);
 static void init_stream_proc(struct packet_stream *);
@@ -85,6 +86,7 @@ static const struct recording_method methods[] = {
 		.sdp_before = sdp_before_proc,
 		.sdp_after = sdp_after_proc,
 		.meta_chunk = meta_chunk_proc,
+		.update_flags = update_flags_proc,
 		.dump_packet = dump_packet_proc,
 		.finish = finish_proc,
 		.init_stream_struct = init_stream_proc,
@@ -243,7 +245,7 @@ static void update_metadata(struct call *call, str *metadata) {
 }
 
 // lock must be held
-static void recording_update_flags(struct call *call) {
+static void update_flags_proc(struct call *call) {
 	append_meta_chunk_null(call->recording, "RECORDING %u", call->recording_on ? 1 : 0);
 	append_meta_chunk_null(call->recording, "FORWARDING %u", call->rec_forwarding ? 1 : 0);
 	for (GList *l = call->streams.head; l; l = l->next) {
@@ -252,6 +254,9 @@ static void recording_update_flags(struct call *call) {
 				ps->unique_id, ps->media->monologue->rec_forwarding ? 1 : 0);
 	}
 }
+static void recording_update_flags(struct call *call) {
+	_rm(update_flags, call);
+}
 
 // lock must be held
 void recording_start(struct call *call, const char *prefix, str *metadata) {
@@ -302,10 +307,13 @@ void recording_start(struct call *call, const char *prefix, str *metadata) {
 
 	recording_update_flags(call);
 }
-void recording_stop(struct call *call) {
+void recording_stop(struct call *call, str *metadata) {
 	if (!call->recording)
 		return;
 
+	if (metadata)
+		update_metadata(call, metadata);
+
 	// check if all recording options are disabled
 	if (call->recording_on || call->rec_forwarding) {
 		recording_update_flags(call);
@@ -346,7 +354,7 @@ void detect_setup_recording(struct call *call, const str *recordcall, str *metad
 	}
 	else if (!str_cmp(recordcall, "no") || !str_cmp(recordcall, "off")) {
 		call->recording_on = 0;
-		recording_stop(call);
+		recording_stop(call, NULL);
 	}
 	else
 		ilog(LOG_INFO, "\"record-call\" flag "STR_FORMAT" is invalid flag.", STR_FMT(recordcall));
@@ -417,7 +425,7 @@ static void sdp_after_pcap(struct recording *recording, GString *str, struct cal
 	// descriptor does not. Make sure to flush any unwritten contents
 	// so the file contents appear in order.
 	if (ml->label.len) {
-		fprintf(meta_fp, "\nLabel: %*s", ml->label.len, ml->label.s);
+		fprintf(meta_fp, "\nLabel: " STR_FORMAT, STR_FMT(&ml->label));
 	}
 	fprintf(meta_fp, "\nTimestamp started ms: ");
 	fprintf(meta_fp, "%.3lf", ml->started.tv_sec*1000.0+ml->started.tv_usec/1000.0);
@@ -448,16 +456,22 @@ static int rec_pcap_meta_finish_file(struct call *call) {
 	time_t start = call->created.tv_sec;
 	time_t end = rtpe_now.tv_sec;
 	char timebuffer[20];
-	struct tm *timeinfo;
+	struct tm timeinfo;
 	struct timeval *terminate;
 	terminate = &(((struct call_monologue *)call->monologues.head->data)->terminated);
 	fprintf(recording->u.pcap.meta_fp, "\nTimestamp terminated ms(first monologue): %.3lf", terminate->tv_sec*1000.0 + terminate->tv_usec/1000.0);
-	timeinfo = localtime(&start);
-	strftime(timebuffer, 20, "%FT%T", timeinfo);
-	fprintf(recording->u.pcap.meta_fp, "\n\ncall start time: %s\n", timebuffer);
-	timeinfo = localtime(&end);
-	strftime(timebuffer, 20, "%FT%T", timeinfo);
-	fprintf(recording->u.pcap.meta_fp, "call end time: %s\n", timebuffer);
+	if (localtime_r(&start, &timeinfo) == NULL) {
+		ilog(LOG_ERROR, "Cannot get start local time, while cleaning up recording meta file: %s", strerror(errno));
+	} else {
+		strftime(timebuffer, 20, "%FT%T", &timeinfo);
+		fprintf(recording->u.pcap.meta_fp, "\n\ncall start time: %s\n", timebuffer);
+	}
+	if (localtime_r(&end, &timeinfo) == NULL) {
+		ilog(LOG_ERROR, "Cannot get end local time, while cleaning up recording meta file: %s", strerror(errno));
+	} else {
+		strftime(timebuffer, 20, "%FT%T", &timeinfo);
+		fprintf(recording->u.pcap.meta_fp, "call end time: %s\n", timebuffer);
+	}
 
 	// Print metadata
 	if (call->metadata.len)
