@@ -1332,8 +1332,8 @@ static void transcode_rr(struct rtcp_process_ctx *ctx, struct report_block *rr)
 	if (!packets)
 		goto out;
 
-	unsigned int lost = atomic64_get(&input_ctx->packets_lost);
-	unsigned int dupes = atomic64_get(&input_ctx->duplicates);
+	unsigned int lost = input_ctx->parent->packets_lost;
+	unsigned int dupes = input_ctx->parent->duplicates;
 	unsigned int tot_lost = lost - dupes; // can be negative/rollover
 
 	ilogs(rtcp, LOG_DEBUG, "Substituting RTCP RR SSRC from %s%x%s to %x: %u packets, %u lost, %u duplicates",
@@ -1449,8 +1449,8 @@ static GString *rtcp_sender_report(struct ssrc_sender_report *ssr,
 	while (rrs->length) {
 		struct ssrc_ctx *s = g_queue_pop_head(rrs);
 		if (i < 30) {
-			struct report_block *rr = (void *) ret->str + ret->len;
-			g_string_set_size(ret, ret->len + sizeof(*rr));
+			g_string_set_size(ret, ret->len + sizeof(struct report_block));
+			struct report_block *rr = (void *) ret->str + ret->len - sizeof(struct report_block);
 
 			// XXX unify with transcode_rr
 
@@ -1467,7 +1467,7 @@ static GString *rtcp_sender_report(struct ssrc_sender_report *ssr,
 			uint32_t jitter = se->jitter;
 			mutex_unlock(&se->h.lock);
 
-			uint64_t lost = atomic64_get(&s->packets_lost);
+			uint64_t lost = se->packets_lost;
 			uint64_t tot = atomic64_get(&s->packets);
 
 			*rr = (struct report_block) {
@@ -1503,6 +1503,7 @@ static GString *rtcp_sender_report(struct ssrc_sender_report *ssr,
 		i++;
 	}
 
+	sr = (void *) ret->str; // reacquire ptr after g_string_set_size
 	sr->rtcp.header.count = n;
 	sr->rtcp.header.length = htons((ret->len >> 2) - 1);
 
@@ -1520,8 +1521,8 @@ static GString *rtcp_sender_report(struct ssrc_sender_report *ssr,
 
 	assert(sizeof(*sdes) == 24);
 
-	sdes = (void *) ret->str + ret->len;
 	g_string_set_size(ret, ret->len + sizeof(*sdes));
+	sdes = (void *) ret->str + ret->len - sizeof(*sdes);
 
 	*sdes = (__typeof(*sdes)) {
 		.sdes.header.version = 2,
